/**************************************************************
*
*                DRAWER.C
*
**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <my_vdi.h>
#include "..\main\main.hpp"   /* CAPTION,VERSION,PLATFORM */

#if defined (__PUREC__)
#   include <acs.h>
#   include <acsplus.h>
#   include <tos.h>
#   include <messages.h>
#   include <acs_plus.h>
#else
#	include "..\midiarea.hpp"
#   include "..\alerts.hpp"
#endif

#include <graphic.h>
#include <notimtab.h>

#include "..\filter\filter.hpp"
#include "..\params\params.hpp"
#include "..\info\info.hpp"

#include "drawer.hpp"
#if defined (__PUREC__)
#   include "drawer.h"
#   include "drawer.ah"
#endif


#if defined (__PUREC__)
/*******************************************************************
*/
	static void keys(Awindow *wi, int kstate, int key)
	
/* Keyboard procedure of the drawer window. See ACS manual.
*
*******************************************************************/
{
	/*** delayed update ***/
	if (set_timer(wi) == -1) Awi_sendall(REDRAW, wi) ;
	
	/*** inform other windows about key pressure ***/
	key_message.sender = wi ;
	key_message.kstate = kstate ;
	key_message.key    = key ;
	Awi_sendall(KEY_PRESS, &key_message) ;
	
	/*** forward event to ACS for further treatment (e.g. by the desktop) ***/
	Awi_keys(wi, kstate, key) ;
}

/*******************************************************************
*/
	static int service(Awindow *wi, int task, void *not_used)
	
/* Service routine for messages sent to this window. See ACS manual.
* 
* OUTPUT:  AS_TERM ==> message is forwarded to parent
*          AS_INFO ==> a little help function 
* 
* RETURN:  TRUE:        if task could be treated
*          FALSE:       else
* 
*******************************************************************/
{
	switch (task)
	{	
		case AS_TERM: Awi_sendall(TERMINATE, wi) ; break ; 
		case AS_INFO: A_dialog(&DRAWER_INFO) ; break ;
		default:      return FALSE ;
	}
	return TRUE ;
}

/*************************/
static void button(void) /*
**************************/
{
	switch (ev_obnr) 
	{
	case FIRST:    select_page(ev_window, first) ; break ;
	case PREVIOUS: select_page(ev_window, previous) ; break ;
	case NEXT:     select_page(ev_window, next) ; break ;
	case LAST:     select_page(ev_window, last) ; break ;
	case ZOOM_IN:  Awi_sendall(ZOOM_PLUS, ev_window) ; break ;
	case ZOOM_OUT: Awi_sendall(ZOOM_MINUS, ev_window) ; break ;
	}
}
#endif

/********************/
static void expand /*
*********************/
(
	char lines_per_12,			/* IN: so many horizontal lines are used per 12 notes */
	NOTE min_note, 				/* IN: marks the minimum of the range to be expanded */
	NOTE max_note, 				/* IN: marks the maximum of the range to be expanded */
	NOTE *note_of_first_line ,	/* OUT: first note of the expanded range */
	char *number_of_lines, 		/* OUT: the expanded range consists of n*(*number_of_lines) notes */
	char *first_modulo_0		/* OUT: indicates the first line with note mod m == 0 */
)
{
	char notes_per_line = 12 / lines_per_12 ;
	*note_of_first_line = min_note / notes_per_line * notes_per_line ;

	*number_of_lines = (max_note - *note_of_first_line + notes_per_line - 1) / 
						notes_per_line + 1 ; 
	*first_modulo_0 = (lines_per_12 - *note_of_first_line % 12 / notes_per_line) %
						lines_per_12 ;
}

/*******************************************************************
*
*             PUBLISHED INTERFACE
*
*******************************************************************/

#if defined (__PUREC__)
void select_page(Awindow *wi, SELECT_MODE sm) 
{
	DRAWER *d = wi->user ;
	char *text ;

	/*** determine page to be drawn ***/
	if (sm == first) d->current_page = 0 ;
	if (sm == previous)
	{
		if (d->current_page > 0) d->current_page -- ;
		else 
		{
			alert_str(FIRST_PAGE, "") ;
			return ;
		}
	}
	if (sm == next)
	{	
		if (d->current_page < d->npgs - 1) d->current_page ++ ;
		else 
		{
			alert_str(LAST_PAGE, "") ;
			return ;
		}
	}
	if (sm == last) d->current_page = d->npgs - 1 ;
	if (d->current_page > d->npgs - 1) d->current_page = d->npgs - 1 ;

	/*** refresh page number in the GEM-object ***/
	text = wi->work[PAGE].ob_spec.tedinfo->te_ptext ;
	if (d->current_page + 1 < 100) 
	{ 
		text[0] = '0' ; 
		text++ ;
	}
	if (d->current_page + 1 < 10) 
	{
		text[0] = '0' ; 
		text++ ;
	}
	itoa( d->current_page + 1, text, 10) ;
	Awi_obchange(wi, PAGE, -1) ;

	/*** draw with new parameters ***/ 
	if (sm != same) redraw(wi) ; 
}


int get_first_page(Awindow *wi)
{ return 0 ; }


int get_last_page(Awindow *wi)
{ return ((DRAWER *)wi->user)->npgs - 1 ; }


static int DRAWER_init(Awindow *wi)
{ 
	select_page(wi, same) ; 
	return OK ;
}

int get_current_page(Awindow *wi)
{ 
DRAWER *d = wi->user ;

	d->current_page = atoi(wi->work[PAGE].ob_spec.tedinfo->te_ptext) - 1 ;
	if (d->current_page > d->npgs - 1) d->current_page = d->npgs - 1 ;
	if (d->current_page < 0          ) d->current_page = 0           ;
	wi->state |= AWS_LATEUPDATE ;	/* call DRAWER_init() later */
	return d->current_page ;
}


void redraw(Awindow *wi)
{
DRAWER *d = wi->user ;

	/*** layout all pages ***/
	if ( page_layouter(d->handle, d->width, d->height, wi) == LAYOUT_ERR )
		return ;
	/*** refresh target object ***/
	Awi_obchange(d->obj_wi, d->obj, (d->obj_wi)->work[d->obj].ob_state) ;
}
#endif

static Awindow *get_info_wi(Awindow *wi)
{ 
#if defined (__PUREC__)
   return ((DRAWER *)(wi->user))->info_wi ; 
#else
   return (Awindow *)&(((MIDI_AREA *)wi)->md) ; 
#endif
}

static DRAWER *get_drawer_struct(Awindow *wi) 
{
#if defined (__PUREC__)
	return (DRAWER *)(wi->user) ;
#else
	return (DRAWER *)(((MIDI_AREA *)wi)->user) ;
#endif
}


static void
draw_system(int handle, Awindow *wi, int x0, int y0, int *points, 
            int zoom, int page, float *y_max, int system, int background) 
{
	float y_min = *y_max ;
	DRAWER *d = get_drawer_struct(wi) ;
	Awindow *params_wi = d->params_wi ; /* to MIDI_OPTIONS */
	Awindow *filter_wi = d->filter_wi ; /* to MIDI_FILTER  */
	PAGE_DESCRIPTOR *p = &d->page[page] ;
	TIME min_time, max_time ;
	NOTE min_note, max_note, note_of_y ;
	char *track_name, text[12] ;
	int track ;
	char number_of_lines, first_shadow ;
	int index_width, dummy ;
	int style     [12], color     [12], 
	    style_ends[12], color_ends[12], 
	    type, mode ;
	char height, dynamic ;
	int back_style, back_color ;
	MARK_MODE mark_mode ;
	int hor_lines, staff_style[12], staff_color[12], 
	    black[12] = { 1,1,1,1,1,1,1,1,1,1,1,1 }, dots ;
	float dyn_scale ;
	int pts[4] ; /* for draw_background */

	/*** fetch note parameters and normalize ***/
	get_style_params(params_wi, 
	                 style     , color, 
	                 style_ends, color_ends, 
	                 &type, &height, &dynamic,
	                 staff_style, staff_color, 
	                 &back_style, &back_color) ;
	if (background && (back_color == WHITE || back_style == 0))
		return ;	/* avoid superflous drawing */
	hor_lines = get_hor_lines(params_wi) ;
	mode      = get_mode     (params_wi) ;	 
	dyn_scale = ( (float)dynamic ) / 10 ;
	vst_effects(handle, NORMAL) ;
		
	min_time = p->min_time + system * d->time_per_system ;
	max_time = min_time + d->time_per_system ;
	for (track = p->first_track ; track <= p->last_track ; track++)
	{
		if ( get_min_max(filter_wi, track, min_time, max_time,
		                 &min_note, &max_note) )
		{
			expand((char)hor_lines, min_note, max_note, 
			         &note_of_y, &number_of_lines, &first_shadow) ;
			if (note_of_y > MAX_NOTE_HEIGHT) note_of_y = 1 ;
			*y_max += d->dy_line * (number_of_lines - 1) ;
			
			/*** if track is visible (inside clipping area) ***/
			if ( ((*y_max + d->dy_line) * zoom + y0 > points[1]) && 
			     ((*y_max - d->dy_line * (number_of_lines + 1)) * zoom + y0 < points[3]) &&
			     !background )
			{
				/*** draw horizontal lines with index and shadow ***/
				vsl_type     (handle, SOLID) ; 
				vst_height   (handle, (int)(d->dy_note * 4 * zoom), &dummy, &dummy, &index_width, &dummy) ;
				vst_alignment(handle, 0, 0, &dummy, &dummy) ; /* left, baseline */
				dots = -1 /* no dots */ ; mark_mode = behind ;
				if      (mode == Rieder) mark_mode = between ;
				else if (mode == Beyreuther)
					dots = d->bars_per_system * (d->bar_divider - 1) ;
				draw_lines(handle, number_of_lines, first_shadow, (char)hor_lines,
				           mark_mode, staff_style, staff_color, (char)((note_of_y + 11)/12), dots,
				           - index_width, 0,
				           x0 + d->x_min_systems * zoom, y0 + (int)(*y_max * zoom + 0.5), 
				           x0 + d->x_max_systems * zoom, y0 + (int)(*y_max * zoom + 0.5),
				           0.0f, - d->dy_line * zoom ) ;
				/*** draw track name ***/
				track_name = get_track_name(filter_wi, track) ;
				vst_alignment(handle, 0, 3, &dummy, &dummy) ;
				vst_color(handle, BLACK) ;
				vswr_mode(handle, MD_TRANS) ;
				v_gtext(handle, x0 + d->x_min_systems * zoom, 
			    	    y0 + (int)((*y_max - (d->dy_line * (number_of_lines - 1) 
			    	          + d->dy_note * 2)) * zoom), track_name ) ;
				/*** draw notes of the current track ***/
				draw_track(handle, filter_wi, track,
				           x0 + d->x_min_systems * zoom, 
				           x0 + d->x_max_systems * zoom, 
				           y0 + (int)(*y_max * zoom + 0.5), 
							- d->dy_note * zoom,
				           min_time, max_time, 
				           note_of_y, 
				           d->dy_note * zoom * height / 4, dyn_scale, 
				           type, style, color, style_ends, color_ends) ;
				if (mode != Beyreuther)
				{
					/*** draw dashed horizontal lines (sub-bars) ***/
#if defined (__PUREC__)
					vsl_udsty(handle, 0x2222) ;
					vsl_type (handle, USERLINE) ;
#else
					vsl_type (handle, DOT) ;
#endif
					draw_lines(handle, (char)(d->bars_per_system * d->bar_divider) + 1, -1, -1,
					           mark_mode, staff_style, black, 0, -1, 0, 0,
					           x0 + d->x_min_systems * zoom, 
					           y0 + (int)(*y_max * zoom + 0.5), 
					           x0 + d->x_min_systems * zoom, 
					           y0 + (int)((*y_max - d->dy_line * (number_of_lines-1) ) * zoom + 0.5),
					           d->dx_bars/d->bar_divider * zoom, 0.0f) ;
				}
			}
			*y_max += d->dy_track ;
		}
	}
	if ( (system != p->number_of_systems - 1) || !(p->type & PRECEEDING) ) 
		*y_max -= d->dy_track ;
	/*** draw solid vertical lines (bars) ***/
	if (background)
	{
		pts[0] = x0 + d->x_min_systems * zoom ;
		pts[1] = y0 + (int)(y_min * zoom + 0.5) ;
		pts[2] = x0 + d->x_max_systems * zoom ;
		pts[3] = y0 + (int)(*y_max * zoom + 0.5) ;
		draw_background(handle, pts, back_style, back_color);
	}
	else
	{
		vsl_type(handle, SOLID) ; 
		draw_lines(handle, d->bars_per_system + 1, -1, -1,
	    	       mark_mode, staff_style, black, 0, -1, 0, 0,
	        	   x0 + d->x_min_systems * zoom, y0 + (int)(y_min * zoom + 0.5), 
	        	   x0 + d->x_min_systems * zoom, y0 + (int)(*y_max * zoom + 0.5),
					d->dx_bars * zoom, 0.0f) ;
		/*** draw bar number ***/
		vswr_mode(handle, MD_TRANS) ;
		vst_alignment(handle, 2, 3, &dummy, &dummy) ;
		vst_color(handle, BLACK) ;
		v_gtext(handle, x0 + d->x_max_systems * zoom, 
	        y0 + (int)(((float)y_min - d->dy_note) * zoom), 
	        itoa((int)(max_time/d->time_per_system * d->bars_per_system), text, 10)) ;
	}
}


void draw_page(int handle, Awindow *wi, int x0, int y0, int *points, 
               int zoom, int page)
{
	DRAWER *d = get_drawer_struct(wi) ;
	Awindow *params_wi = d->params_wi ; /* to MIDI_OPTIONS */
	PAGE_DESCRIPTOR *p = &d->page[page] ;
	char text[12], footline[120] ;
	unsigned long _time ;
	float y, y_dummy ;
	int system ;
	int dummy ;
#if defined (__PUREC__)
	long filesize = get_file_size(d->info_wi) ;
#else
	long filesize = ((MIDI_AREA *)wi)->md.length ;
#endif

	if (d->npgs == 0) return ;
	vst_font (handle, d->font_id) ;

	/*** draw title ***/
	if ( page == 0 && y0 + d->y_min_systems_0 * zoom > points[1] )
	{
		vst_color(handle, BLACK) ;
		vst_height(handle, (int)(d->dy_system * zoom + 0.5)/HEADLINE_DIVIDER, 
		           &dummy, &dummy, &dummy, &dummy) ;
		vst_alignment(handle, 1, 2, &dummy, &dummy) ;
		vst_effects(handle, get_text_effects(params_wi)) ;
		v_gtext(handle, 
				x0 + ((d->x_min + d->x_max)>>1) * zoom, 
				y0 + d->y_min * zoom, 
		        get_title(params_wi)) ;
	}
	/*** draw note system(s) ***/
	if (page == 0)
	     y = (float)(d->y_min_systems_0) ;
	else y = (float)(d->y_min_systems  ) ;
	if (p->type & SUCCEEDING) 
		y += d->dy_track ;
	for (system = 0 ; system < p->number_of_systems ; system ++)
	{
		y_dummy = y ;
		draw_system(handle, wi, x0,y0, points, zoom, page, 
			&y_dummy, system, TRUE) ; /* background only */
		draw_system(handle, wi, x0,y0, points, zoom, page, 
			&y, system, FALSE) ; /* foreground only */
		y += d->dy_system ; 
	}
	/*** draw foot line ***/
	if (y0 + d->y_max_systems * zoom < points[3])
	{
		vst_color  (handle, BLACK) ;
		vst_effects(handle, NORMAL) ;
		vst_height (handle, (int)(d->dy_note * 4 * zoom), &dummy,&dummy,&dummy,&dummy) ;
		
		/*** left part ***/
		/* filename and size */
		sprintf(footline, "%s (%ld KByte) printed on ", d->filename, filesize) ;
		/* year month day */
#if defined (__PUREC__)
		_time = Gettime() ;
		itoa((int)(_time>>25) + 1980, text, 10) ; /* year */
#else
		SYSTEMTIME time ;
		GetLocalTime(&time) ;
		itoa(time.wYear, text, 10) ;
#endif
		strcat(footline, text) ; 
		strcat(footline, "-") ;
#if defined (__PUREC__)
		itoa((int)(_time>>21) & 0x0F, text, 10) ; /* month */
#else
		itoa(time.wMonth, text, 10) ;
#endif
		strcat(footline, text) ; 
		strcat(footline, "-") ;
#if defined (__PUREC__)
		itoa((int)(_time>>16) & 0x1F, text, 10) ; /* day */
#else
		itoa(time.wDay, text, 10) ;
#endif
		strcat(footline, text) ; 

		/* caption, version, platform */
		sprintf(footline + strlen(footline), 
			" by%sV%s for %s", CAPTION, VERSION, PLATFORM) ;
		vst_alignment(handle, 0, 4, &dummy, &dummy) ; /* left, char. bottom */
		v_gtext(handle, x0 + d->x_min * zoom, y0 + d->y_max * zoom, footline) ;
		
		/*** right part ***/
		sprintf(footline, "page %d of %d", page+1, d->npgs) ;		
		vst_alignment(handle, 2, 4, &dummy, &dummy) ; /* right, char. bottom */
		v_gtext(handle, x0 + d->x_max * zoom, y0 + d->y_max * zoom, footline) ;
	}	
}


int page_layouter(int handle, int width, int height, Awindow *wi)
{
#if defined (__PUREC__)
	DRAWER *d = wi->user ;
#else
	DRAWER *d = (DRAWER *)(((MIDI_AREA *)wi)->user) ;
#endif
	Awindow *params_wi = d->params_wi ; /* to MIDI_OPTIONS */
/*	Awindow *filter_wi = d->filter_wi ;    to MIDI_FILTER  */
	Awindow *info_wi   = (Awindow *)get_info_wi(wi) ;   /* to MIDI_DATA */
	int index_width, index_height, headline_width, headline_height ; 
	char number_of_lines, first_modulo_0 ;
	int first_active_track, last_active_track ;
	PAGE_DESCRIPTOR *page ;   
	int p ;                   /* page number */
	int t ;                   /* track number */
	float y ;                 /* current y value */
	NOTE min_note, max_note, note_of_y ;
	TIME time, max_time ;
	int dummy, hor_lines ;

	/*** get maximum area allowed for whole drawing ***/
	d->x_min = (int)(((long)(width - 1)) * 
	                  get_left_border(params_wi) / 100) ;
	d->x_max = (int)(((long)(width - 1)) * 
	                  (100 - get_right_border(params_wi)) / 100) ;
	d->y_min = (int)(((long)(height - 1)) * 
	                  get_upper_border(params_wi) / 100) ;
	d->y_max = (int)(((long)(height - 1)) * 
	                  (100 - get_lower_border(params_wi)) / 100) ;

	/*** check for plausability ***/
	if ( (d->x_min >= d->x_max) || (d->y_min >= d->y_max) )
	{
		alert_str(INVALID_BORDER_VALUES, "") ;
		d->npgs = 0 ;
		return LAYOUT_ERR ; 
	}

	/*** times, bars ***/
	d->beats_per_system = ((float)get_system_length(params_wi))/2 ;
	d->time_per_system = (__int32)( d->beats_per_system * 
	                     get_ticks_per_beat(info_wi) ) ;
	d->bars_per_system = get_bars_per_system(params_wi) ;
	d->bar_divider = get_bar_divider(params_wi) ;

	/*** get y-distances ***/
	d->dy_system = ((float)height * get_system_distance(params_wi)) / 100 ;
	d->dy_track  = ((float)height * get_track_distance (params_wi)) / 100 ;
	d->dy_note   = ((float)height * get_note_distance  (params_wi)) / 1000 ;
	hor_lines = get_hor_lines(params_wi) ;
	switch (hor_lines)
	{
	case 1:  d->dy_line = d->dy_note * 12 ; break ;
	case 2:  d->dy_line = d->dy_note * 6 ; break ;
	case 3:  d->dy_line = d->dy_note * 4 ; break ;
	case 4:
	case 5:  d->dy_line = d->dy_note * 3 ; break ;
	default: d->dy_line = d->dy_note * 2 ;
	}

	/*** get font and its attributes ***/
	d->font_id = get_font_id(params_wi) ;
	if ( d->font_id != vst_font(handle, d->font_id) )
	{
		alert_str(FONT_NOT_EXISTING, "") ;
		d->npgs = 0 ;
		return LAYOUT_ERR ; 
	} 
	/*** note: actual index_height needs not to be desired height ***/
	vst_height(handle, (int)(4 * d->dy_note), 
	           &dummy, &dummy, &index_width, &index_height) ; /* cell size */
	/*** note: actual headline_height needs not to be HEADLINE_HEIGHT ! ***/
	vst_height(handle, (int)(d->dy_system/HEADLINE_DIVIDER), 
	          &headline_width, &headline_height, &dummy, &dummy) ; /* char size */
	
	/*** get maximum area allowed for drawing note systems ***/
	d->x_min_systems   = d->x_min + index_width ;
	d->x_max_systems   = d->x_max ;
	d->y_min_systems_0 = (int)(d->dy_system + d->y_min + headline_height/2 + 0.5) ;
	d->y_min_systems   = d->y_min ;	/* all pages except the 0th */
	d->y_max_systems   = d->y_max ;

	/*** get x-distances ***/
	d->dx_bars = ((float)(d->x_max_systems - d->x_min_systems)) /
	              d->bars_per_system ;

	/*** tracks ***/
	first_active_track = get_first_active_track(d->filter_wi) ;
	last_active_track = get_last_active_track(d->filter_wi) ;
	if ( (first_active_track < 0) || (last_active_track < 0) )
	{
		alert_str(ALL_TRACKS_OFF, "") ;
		d->npgs = 0 ;
		return LAYOUT_ERR ;
	}

	/*** layout all pages ***/
	p = 0 ; d->npgs = 1 ; 
	page = d->page ;
	page[p].min_time = 0 ;
	page[p].type = NO_TYPE ;
	page[p].number_of_systems = 0 ;
	page[p].first_track = first_active_track ; 
	t = first_active_track ;
	y = (float)(d->y_min_systems_0) ;
	time = 0 ;
	max_time = get_absolute_max_time(d->filter_wi) ;

	while ( (time < max_time) && (p < MAX_PAGES - 1) ) 
	{
		/*** layout one system ***/
		while ( (t <= last_active_track) && 
		        (time < max_time) && (p < MAX_PAGES - 1) )
		{
			if (page[p].type & SUCCEEDING) y += d->dy_track ;
			while ( (y <= d->y_max_systems) && (t <= last_active_track) )
			{
				/*** if this track is active ***/
				if (get_min_max(d->filter_wi, t, 
				                time, time + d->time_per_system,
				                &min_note, &max_note))
				{
					expand((char)hor_lines, min_note, max_note, 
					         &note_of_y, &number_of_lines, &first_modulo_0) ;
					/*** fits each track into the allowed y-range of a page ? ***/
					if (d->dy_line * (number_of_lines - 1) > 
				    d->y_max_systems - d->y_min_systems - 2 * d->dy_track)
					{
						alert_str(TRACK_TOO_HIGH, "") ;
						d->npgs = p ;	
						return LAYOUT_ERR ;
					}
					y += d->dy_line * (number_of_lines - 1) ;
					y += d->dy_track ;
				}
				t ++ ;
			}
			t -- ;
			if (y > d->y_max_systems)
			{
				/*** the current note system does not fit onto this page ***/
				y = (float)(d->y_min_systems) ;
				d->npgs ++ ;
				if (page[p].number_of_systems == 0)
				{
					/*** this is the first system for this page ***/
					page[p].type |= PRECEEDING ; 
					page[p].last_track = t - 1 ; 
					page[p].number_of_systems = 1 ; /* but not a whole system */
					p ++ ; 
					page[p].type = SUCCEEDING ;
					page[p].first_track = t ;
				}
				else 
				{
					/*** there are already one or more systems on this page ***/
					p ++ ; 
					page[p].type = NO_TYPE ;
					t = page[p].first_track = first_active_track ;
				}
				page[p].number_of_systems = 0 ;
				page[p].min_time = time ;
			}
			else
			{
				/*** the current note system fits onto this page ***/
				time += d->time_per_system ;
				t = first_active_track ;
				if (page[p].type == SUCCEEDING)
				{
					/*** one and only system for this page, take a new page ***/
					page[p].last_track = last_active_track ; 
					page[p].number_of_systems = 1 ; /* but not a whole system */
					y = (float)(d->y_min_systems) ;
					p ++ ; if (time < max_time) d->npgs ++ ;
					page[p].type = NO_TYPE ;
					page[p].first_track = first_active_track ;
					page[p].last_track = first_active_track ;
					page[p].number_of_systems = 0 ;
					page[p].min_time = time ;
				}
				else
				{
					/*** one more system for this page ***/
					page[p].type = MULTIPLE ;
					page[p].last_track = last_active_track ;
					page[p].number_of_systems ++ ;
					y += (d->dy_system - d->dy_track) ;
					page[p+1].type = NO_TYPE ;
					page[p+1].first_track = first_active_track ;
				}
			}
		}
	}
	if (d->current_page > d->npgs - 1) d->current_page = d->npgs - 1 ;
	if (d->npgs >= MAX_PAGES - 1)
	{
		alert_str(MAX_PAGES_REACHED, "" ) ;
	}
	return LAYOUT_GOOD ;
}

#if defined (__PUREC__)
void reset_layout(Awindow *wi)
{ 
	DRAWER *d = wi->user ;
	page_layouter(d->handle, d->width, d->height, wi) ;
}
					
		
void DRAWER_destructor(Awindow *wi)
{
	if (wi) 
	{
		memory_symptom = "DrDstr wi->user" ;
		if (wi->user) My_free(wi->user) ;
		memory_symptom = "DrDstr wi" ;
		Awi_delete(wi) ;
	}
}


Awindow *DRAWER_constructor(DRAWER_INIT *drawer_init)
{
Awindow *wi ;
DRAWER *d ;

	/*** enter filename into icon text and info line ***/
	DRAWER_WINDOW.iconblk = drawer_init->icon ;
	DRAWER_WINDOW.iconblk->monoblk.ib_ptext = drawer_init->filename ;
	DRAWER_WINDOW.info = drawer_init->filename ;

	/*** create the DRAWER window ***/
	wi = Awi_create(&DRAWER_WINDOW) ;
	if (!wi) return NULL ;

	/*** allocate drawer structure and initialize ***/
	wi->user = d = My_alloc(sizeof *d) ;
	if (!d) { DRAWER_destructor(wi) ; return NULL ;	}
	d->handle    = drawer_init->handle ;
	d->obj_wi    = drawer_init->obj_wi ;
	d->obj       = drawer_init->obj ;
	d->width     = drawer_init->width ;
	d->height    = drawer_init->height ;
	d->params_wi = drawer_init->params_window ;
	d->filter_wi = drawer_init->filter_window ;
	d->info_wi   = drawer_init->info_window ;
	d->filename  = drawer_init->filename ;
	d->ntrks = get_number_of_tracks(drawer_init->info_window) ;
	d->current_page = 0 ; 
	d->npgs         = 0 ;

	/*** draw at once ***/
	redraw(wi) ; 

	/*** return success message ***/
	return wi ;
}
#endif
