/**************************************************************
*
*                PARAMS.C
*
**************************************************************/

#if defined (__PUREC__)
#	include <my_vdi.h>
#	include <acs.h>
#	include <acsplus.h>
#	include <tos.h>
#	include <messages.h>
#	include <diskfile.h>
#else 
#	include "..\midifilt.hpp"
#	include "..\midiopt.hpp"
#endif
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <acs_plus.h>
#include <graphic.h>

#include "params.hpp"

/** net length of title field in params object (without '\0') **/
#define TITLE_FIELD_LENGTH 42

/** length of title field in *.par file (with '\0') **/
#define TITLE_FIELD_LENGTH_FILE 80 /* must be > TITLE_FIELD_LENGTH */

#if defined (__PUREC__)
#	include "params.h"
#	include "params.ah"

static char default_path[pathname_length] ; /** for MIDI_PRT.PAR **/

/*** paths where to search for files, for all instances together ***/
static char savepath[pathname_length] ;
static char loadpath[pathname_length] ;
static char savepathname[pathname_length] ;
static char loadpathname[pathname_length] ;

/*** array for addressing menu entries in program loops, ***/
/*** FONT_i are not defined in sequence by ACS !          ***/

static char FONT_INDEX[MAX_FONTS] = 
{ 
	FONT_0  , FONT_1  , FONT_2  , FONT_3  , FONT_4  , FONT_5  , FONT_6  , FONT_7  , FONT_8  , FONT_9  , 
	FONT_10 , FONT_11 , FONT_12 , FONT_13 , FONT_14 , FONT_15 , FONT_16 , FONT_17 , FONT_18 , FONT_19 , 
	FONT_20 , FONT_21 , FONT_22 , FONT_23 ,	FONT_24 , FONT_25 , FONT_26 , FONT_27 , FONT_28 , FONT_29 , 
	FONT_30 , FONT_31 , FONT_32 , FONT_33 ,	FONT_34 , FONT_35 , FONT_36 , FONT_37 , FONT_38 , FONT_39 , 
	FONT_40 , FONT_41 , FONT_42 , FONT_43 ,	FONT_44 , FONT_45 , FONT_46 , FONT_47 , FONT_48 , FONT_49 , 
	FONT_50 , FONT_51 , FONT_52 , FONT_53 ,	FONT_54 , FONT_55 , FONT_56 , FONT_57 , FONT_58 , FONT_59 , 
	FONT_60 , FONT_61 , FONT_62 , FONT_63 ,	FONT_64 , FONT_65 , FONT_66 , FONT_67 , FONT_68 , FONT_69 , 
	FONT_70 , FONT_71 , FONT_72 , FONT_73 ,	FONT_74 , FONT_75 , FONT_76 , FONT_77 , FONT_78 , FONT_79 , 
	FONT_80 , FONT_81 , FONT_82 , FONT_83 ,	FONT_84 , FONT_85 , FONT_86 , FONT_87 , FONT_88 , FONT_89 , 
	FONT_90 , FONT_91 , FONT_92 , FONT_93 ,	FONT_94 , FONT_95 , FONT_96 , FONT_97 , FONT_98 , FONT_99 , 
	FONT_100, FONT_101, FONT_102, FONT_103,	FONT_104, FONT_105, FONT_106, FONT_107, FONT_108, FONT_109, 
	FONT_110, FONT_111, FONT_112, FONT_113,	FONT_114, FONT_115, FONT_116, FONT_117, FONT_118, FONT_119, 
	FONT_120, FONT_121, FONT_122, FONT_123,	FONT_124, FONT_125, FONT_126, FONT_127
} ;

#endif

#ifndef __PUREC__
#	pragma pack(1)
#endif
typedef struct
{
	/*** headline ***/
	char title[TITLE_FIELD_LENGTH_FILE] ;
	/*** text parameters ***/
	int font_id, effect ;
	/*** borders ***/
	char left_border, right_border, lower_border, upper_border ;
	/*** y-distances ***/
	char system_distance, track_distance, note_distance ;
	/*** bar parameters ***/ 
	char bars_per_system, bar_length, sub_bars ;
	/*** vertical parameters ***/
	int track_style[12], track_color[12], hor_lines, mode ;
	/*** note parameters ***/
	int note_style[12], note_color[12], note_type ;
	char note_height, note_dynamic ;
} 
PARAMS_FILE_V2 ; /* old version */

typedef struct
{
	/*** headline ***/
	char title[TITLE_FIELD_LENGTH_FILE] ;
	/*** text parameters ***/
	int font_id, effect ;
	/*** borders ***/
	char left_border, right_border, lower_border, upper_border ;
	/*** y-distances ***/
	char system_distance, track_distance, note_distance ;
	/*** bar parameters ***/ 
	char bars_per_system, bar_length, sub_bars ;
	/*** vertical parameters ***/
	int track_style[12], track_color[12], hor_lines, mode ;
	/*** note parameters ***/
	int note_style[12], note_color[12], note_type ;
	char note_height, note_dynamic ;
	/*** others, new in V3 ***/
	int back_style, back_color ;
	/*** note parameters, new in V3 ***/
	int note_style_ends[12], note_color_ends[12] ;
} 
PARAMS_FILE ;
#ifndef __PUREC__
#	pragma pack()
#endif

#if defined (__PUREC__)
/************* OBJECT DATA PROTOTYPE BEGIN ***********************/

typedef struct 
{
	char path[pathname_length] ;
	int  note_style[12]     , note_color[12] ;
	int  note_style_ends[12], note_color_ends[12] ;
	int track_style[12]     ,track_color[12], track_index ;
	int  back_style         , back_color ;
	int *style_to_update, *color_to_update ; /* for modal dialogs */
	int handle ;
	int effect ;
	int font_id ;
	FONT_INFO *font_info ;     /* info about available GDOS fonts */
	int number_fonts ;         /* number of elements in font_info[] */
	Awindow *font_wi ;         /* pointer to font selection window */
	Awindow *params_wi ;       /* pointer to the parameters window */
} 
PARAMS ;

/************* OBJECT DATA PROTOTYPE END *************************/


/************************/
static void draw_cross /*
*************************/
(
	int handle,
	int points[4],
	int color
)
{
	int points3 = points[3] ;
	vswr_mode(handle, MD_REPLACE) ;
	vsl_color(handle, color) ;
	vsl_width(handle, 1) ;
	v_pline(handle, 2, points) ;
	/*** swap ***/
	points[3] = points[1] ;
	points[1] = points3 ;
	v_pline(handle, 2, points) ;
	/*** swap back ***/
	points[1] = points[3] ;
	points[3] = points3 ;
}


/**********************/
static void button() /*
***********************/
{
	PARAMS *p = ev_window->user ;
	int buff_note_color     , buff_note_style, 
	    buff_note_color_ends, buff_note_style_ends ;
	int buff ;
	int i ;
	int delta = 1 ;
	int *colors, modulo ;
    
    switch (ev_obnr)
    {
    case SWAP:
    	/*** swap track colors and styles ***/
    	for (i = 0 ; i < 5 ; i++)
    	{
    		buff = p->track_color[i] ;
    		p->track_color[i] = p->track_color[0xA - i] ;
    		p->track_color[0xA - i] = buff ;
    		
    		buff = p->track_style[i] ;
    		p->track_style[i] = p->track_style[0xA - i] ;
    		p->track_style[0xA - i] = buff ;
    	}
    	break ;
    	
    case TRANSPOSE:
    	/*** rotate note colors and styles ***/
    	buff_note_color = p->note_color[0xB] ;
    	buff_note_style = p->note_style[0xB] ;

    	buff_note_color_ends = p->note_color_ends[0xB] ;
    	buff_note_style_ends = p->note_style_ends[0xB] ;

    	for (i = 0xB ; i > 0 ; i--)
    	{
    		p->note_color[i] = p->note_color[i-1] ;
    		p->note_style[i] = p->note_style[i-1] ;
    		
    		p->note_color_ends[i] = p->note_color_ends[i-1] ;
    		p->note_style_ends[i] = p->note_style_ends[i-1] ;
    	}
    	p->note_color[0] = buff_note_color ;
    	p->note_style[0] = buff_note_style ;

    	p->note_color_ends[0] = buff_note_color_ends ;
    	p->note_style_ends[0] = buff_note_style_ends ;

    	break ;

    case DECR_NOTE: 
    case DECR_END: 
    case DECR_TRACK:
    	delta = -1 ;
    case INCR_NOTE: 
    case INCR_END: 
    case INCR_TRACK:
		if ( (ev_object[ev_obnr].ob_state) & AOS_DCLICK && 
		      ncolors > 16 )
			delta *= 16 ;
		switch (ev_obnr)
		{
	    case INCR_NOTE: 
	    case DECR_NOTE:   colors = p->note_color      ; break ;
	    case INCR_END: 
	    case DECR_END:    colors = p->note_color_ends ; break ;
	    case INCR_TRACK:
	    case DECR_TRACK:  colors = p->track_color     ; break ;
		}
		modulo = ncolors ;
		if (modulo > 256) modulo = 256 ;
		for (i = 0 ; i < 12 ; i++)
			colors[i] = (modulo + colors[i] + delta) % modulo ;
    }
	Awi_obchange(ev_window, EXAMPLE, -1) ;	        
	if (set_timer(ev_window) == -1) Awi_sendall(REDRAW, ev_window) ;
}


/***********************/
static void example() /*
************************/
{
    PARAMS *p = ev_window->user ;
	int index_x, index_y, fine_index ;
	Axywh offset ;
	OBJECT *obj ; int ob_index ;

	Aob_offset(&offset, ev_object, EXAMPLE) ;

	index_x =      (ev_mmox - offset.x - ev_window->wi_work.x) * 12 / offset.w ;
	index_y = 11 - (ev_mmoy - offset.y - ev_window->wi_work.y) * 12 / offset.h ;
	if (index_x == index_y)
	{
		/*** is a note ***/
		fine_index = (ev_mmox - offset.x - ev_window->wi_work.x) * 36 / offset.w ;
		if (fine_index % 3 == 1)
		{
			/* is a note body */
			p->color_to_update = &(p->note_color[index_x]) ;
			p->style_to_update = &(p->note_style[index_x]) ;
		}
		else
		{
			/* is a head or tail */
			p->color_to_update = &(p->note_color_ends[index_x]) ;
			p->style_to_update = &(p->note_style_ends[index_x]) ;
		}
	}
	else
	{
		fine_index = (ev_mmoy - offset.y - ev_window->wi_work.y) * 24 / offset.h ;
		if (fine_index <= 16) 
		{
			/*** in the upper area => background ***/
			p->color_to_update = &(p->back_color) ;
			p->style_to_update = &(p->back_style) ;
		}
		else if (fine_index < 23) 
		{
			/*** in the middle => current track ***/
			p->color_to_update = &(p->track_color[p->track_index]) ;
			p->style_to_update = &(p->track_style[p->track_index]) ;
		}
		else
		{
			/*** in the lower area => track depending on x pos. ***/
			index_x = (ev_mmox - offset.x - ev_window->wi_work.x) * 11 / offset.w ;
			p->color_to_update = &(p->track_color[index_x]) ;
			p->style_to_update = &(p->track_style[index_x]) ;
		}
	}
	if ( (ev_object[ev_obnr].ob_state) & AOS_DCLICK )
	{
		obj = Aob_create(&PATTERN_WORK) ;
		ob_index = PATTERN ;
	}
	else 
	{
		obj = Aob_create(&COLOR_WORK) ;
		ob_index = COLOR ;
	}
	if (obj) 
	{
		obj[ob_index].ob_spec.userblk->ub_parm = (long)p ;
		A_dialog(obj) ;
		Aob_delete(obj) ;
	}	
	Awi_obchange(ev_window, EXAMPLE, -1) ;	        
	if (set_timer(ev_window) == -1) Awi_sendall(REDRAW, ev_window) ;
}

/*************************************************/
static int cdecl example_draw(PARMBLK *parmblk) /*
**************************************************/
{
	PARAMS *p = (PARAMS *)parmblk->pb_parm ;
	int i, points[4], clipping_points[4] ;
	float dy_notes  = ((float)parmblk->pb_h) / 12 ,
		  dx_notes  = ((float)parmblk->pb_w) / 12 ,
		  dx_tracks = ((float)parmblk->pb_w) / 11 ;
		  
		  
	int note_style[12]     , note_color[12]     ,
	    note_style_ends[12], note_color_ends[12],
	    note_type,
	    track_style[12]    , track_color[12]    ,
	    back_style         , back_color         ;
	char note_height, note_dynamic ;
	
	get_style_params(p->params_wi,
		note_style     ,note_color     ,
		note_style_ends,note_color_ends,
		&note_type,	 &note_height, &note_dynamic, 
		track_style    ,track_color    ,
		&back_style    ,&back_color    
	);

	/*** calculate border points for clipping rectangle, clipping = on ***/
	clipping_points[0] = parmblk->pb_xc ;	
	clipping_points[1] = parmblk->pb_yc ;	
	clipping_points[2] = clipping_points[0] + parmblk->pb_wc - 1 ;	
	clipping_points[3] = clipping_points[1] + parmblk->pb_hc - 1 ;	
	vs_clip(p->handle, TRUE, clipping_points) ;

	/*** draw background ***/
	/* workaround-frame in white */
	points[0] = parmblk->pb_x - 3 ;
	points[1] = parmblk->pb_y - 2 ;	
	points[2] = points[0] + parmblk->pb_w + 5 ;
	points[3] = points[1] + parmblk->pb_h + 3 ;	
	draw_background(p->handle, points, FIS_SOLID, WHITE) ;
	/* the background */
	points[0] = parmblk->pb_x ;
	points[1] = parmblk->pb_y ;	
	points[2] = points[0] + parmblk->pb_w - 1 ;
	points[3] = points[1] + parmblk->pb_h - 1 ;	
	draw_background(p->handle, points, back_style, back_color) ;
	
		
	/*** draw hor. lines ***/
	points[1] = parmblk->pb_y + (int)(dy_notes * 11.5) ;	
	points[2] = points[0] + parmblk->pb_w /*- 1*/ ;
	draw_lines(p->handle, 4 /*number_of_lines*/, 0 /*first_shadow*/, 
	           4 /*hor_lines*/,
	           between /*mark_mode*/, 
	           track_style, track_color, p->track_index, 
	           -1 /*no dots*/, 
	           INVALID_DX,INVALID_DY, /*no text*/
	           points[0], points[1], points[2], points[1],
	           0.0f, - dy_notes * 3) ;
	/*** draw 11 track colors & styles ***/
	for (i = 0 ; i < 11 ; i++)
		draw_lines(p->handle, 2 /*number_of_lines*/, 0 /*first_shadow*/, 
	           4 /*hor_lines*/,
	           between /*mark_mode*/, 
	           track_style, track_color, i, 
	           -1 /*no dots*/, 
	           INVALID_DX,INVALID_DY, /*no text*/
	           points[0] + (int)(dx_tracks * i)    , points[1], 
	           points[0] + (int)(dx_tracks * (i+1)), points[1],
	           0.0f, dy_notes/2) ;
	
	/*** draw notes ***/
	for (i = 0 ; i < 12 ; i++)
	{
		points[0] = parmblk->pb_x + (int)(dx_notes * i) ;
		points[1] = parmblk->pb_y + (int)(dy_notes * (11.75f - i) + 0.5) ;	
		points[2] = points[0] + (int)(dx_notes) ;
		points[3] = points[1] - (int)(dy_notes/2 + 0.5) ;	
		draw_note(p->handle, points, 
			note_type,note_style[i],note_color[i],
			note_style_ends[i],note_color_ends[i] ) ;
	}
	/*** clipping = off ***/
	vs_clip(p->handle, FALSE, clipping_points) ;
	
	return(parmblk->pb_currstate) ;          
}

/***********************/
static void pattern() /*
************************/
{
    PARAMS *p = ev_window->user ;
	Axywh offset ;
	int number_patterns = 9 ;

	Aob_offset(&offset, ev_object, COLOR) ;

	/*** update style ("pattern") ***/
	*(p->style_to_update) = 
		(ev_mmox - offset.x - ev_window->wi_work.x) * number_patterns / offset.w ;

	dia_abort = TRUE ; /* abort dialog */
}


/**************************************************/
static int cdecl pattern_draw(PARMBLK *parmblk)  /*
***************************************************/
{
	PARAMS *p = (PARAMS *)parmblk->pb_parm ;
	int points[4], clipping_points[4], i ;
	int number_patterns = 9, color ;
	float dx = ((float)parmblk->pb_w) / number_patterns ;

	/*** calculate border points for clipping rectangle, clipping = on ***/
	clipping_points[0] = parmblk->pb_xc ;	
	clipping_points[1] = parmblk->pb_yc ;	
	clipping_points[2] = clipping_points[0] + parmblk->pb_wc - 1 ;	
	clipping_points[3] = clipping_points[1] + parmblk->pb_hc - 1 ;	
	vs_clip(p->handle, TRUE, clipping_points) ;

	/*** fit points to userdef object size and location ***/
	points[1] = parmblk->pb_y ;	
	points[3] = parmblk->pb_y + parmblk->pb_h - 1 ;	

	/*** draw pattern fields ***/
	for ( i = 0 ; i < number_patterns ; i++ )
	{
		points[0] = parmblk->pb_x + (int)(dx * i) ;
		points[2] = points[0] + (int)(dx);
		draw_note(p->handle, points, BODY_RECT, i, BLACK, 0,0) ;
		if ( i == *(p->style_to_update) )
		{
			/*** draw cross in "inverted" color ***/
			color = i > 3 ? WHITE : BLACK ;
			draw_cross(p->handle, points, color) ;
		}
	}
	/*** clipping = off ***/
	vs_clip(p->handle, FALSE, clipping_points) ;
	
	return(parmblk->pb_currstate) ;          
}


/*************************/
static void color(void) /*
**************************/
{
    PARAMS *p = ev_window->user ;
	Axywh offset ;
	int root = sqrt(ncolors) ;

	Aob_offset(&offset, ev_object, COLOR) ;

	/*** update color ***/
	if (ncolors == 2)
		*(p->color_to_update) = 
			(ev_mmox - offset.x - ev_window->wi_work.x) * 2 / offset.w ;
	else
		*(p->color_to_update) = 
			(ev_mmox - offset.x - ev_window->wi_work.x) * root / offset.w +
	    	(((ev_mmoy - offset.y - ev_window->wi_work.y) * root / offset.h) * root ) ;

	dia_abort = TRUE ; /* abort dialog */
}


/***********************************************/
static int cdecl color_draw(PARMBLK *parmblk) /*
************************************************/
{
	PARAMS *p = (PARAMS *)parmblk->pb_parm ;
	int points[4], clipping_points[4], i,j ;
	int root = sqrt(ncolors) ; 
	int rgb[3], color ;
	float dx = ((float)parmblk->pb_w) / root ;
	float dy = ((float)parmblk->pb_h) / root ;

	/*** calculate border points for clipping rectangle, clipping = on ***/
	clipping_points[0] = parmblk->pb_xc ;	
	clipping_points[1] = parmblk->pb_yc ;	
	clipping_points[2] = clipping_points[0] + parmblk->pb_wc - 1 ;	
	clipping_points[3] = clipping_points[1] + parmblk->pb_hc - 1 ;	
	vs_clip(p->handle, TRUE, clipping_points) ;

	/*** draw color fields ***/
	if (ncolors == 2)
	{
		/*** draw left half white and right half black ***/
		points[0] = parmblk->pb_x ;
		points[1] = parmblk->pb_y ;
		points[2] = points[0] + (parmblk->pb_w - 1) / 2 ;
		points[3] = points[1] + parmblk->pb_h - 1 ;
		draw_note(p->handle, points, BODY_RECT, IP_SOLID, WHITE, 0,0) ;
		if ( WHITE == *(p->color_to_update) )
			draw_cross(p->handle, points, BLACK) ;
		points[0] = parmblk->pb_x + parmblk->pb_w - 1 ;
		draw_note(p->handle, points, BODY_RECT, IP_SOLID, BLACK, 0,0) ;
		if ( BLACK == *(p->color_to_update) )
			draw_cross(p->handle, points, WHITE) ;
	}
	else for ( j = 0 ; j < root ; j++ )
	{
		points[1] = parmblk->pb_y + (int)(dy * j) + 2 ;
		points[3] = points[1] + (int)(dy) - 2 ;
		for ( i = 0 ; i < root ; i++ )
		{
			points[0] = parmblk->pb_x + (int)(dx * i) + 1 ;
			points[2] = points[0] + (int)(dx) - 3 ;
			color = (j*root)+i ;
			draw_note(p->handle, points, BODY_RECT, IP_SOLID, color, 0,0) ;

			if ( color == *(p->color_to_update) )
			{
				/*** draw cross in "inverted" color ***/
				vq_color(p->handle, color, FALSE, rgb) ;
				#define COL_LIMIT 600
				if ( rgb[0] > COL_LIMIT || 
				     rgb[1] > COL_LIMIT || 
				     rgb[2] > COL_LIMIT )
	     		     color = BLACK ;
				else color = WHITE ;
				draw_cross(p->handle, points, color) ;
			}
		}		
	}
	/*** clipping = off ***/
	vs_clip(p->handle, FALSE, clipping_points) ;
	
	return(parmblk->pb_currstate) ;          
}

/**********************************************/
static int cdecl note_draw(PARMBLK *parmblk) /*
***********************************************/
{
	PARAMS *p = (PARAMS *)parmblk->pb_parm ;
	int points[4], clipping_points[4] ;
	int type = 0 ;

	/*** calculate border points for clipping rectangle, clipping = on ***/
	clipping_points[0] = parmblk->pb_xc ;	
	clipping_points[1] = parmblk->pb_yc ;	
	clipping_points[2] = clipping_points[0] + parmblk->pb_wc - 1 ;	
	clipping_points[3] = clipping_points[1] + parmblk->pb_hc - 1 ;	
	vs_clip(p->handle, TRUE, clipping_points) ;

	/*** fit points to userdef object size and location ***/
	points[0] = parmblk->pb_x ;	
	points[1] = parmblk->pb_y + (parmblk->pb_h >> 2) ;	
	points[2] = parmblk->pb_x + parmblk->pb_w - 1 ;	
	points[3] = parmblk->pb_y + parmblk->pb_h - 1 - (parmblk->pb_h >> 2) ;	

	/*** draw corresponding note ***/
	switch(parmblk->pb_obj)
	{
		case HEAD_0: break ;
		case HEAD_1: type = HEAD_LINE ; break ;
		case HEAD_2: type = HEAD_TRI  ; break ;
		case HEAD_3: type = HEAD_DOT  ; break ;
		case BODY_0: break ;
		case BODY_1: type = BODY_RECT  ; break ;
		case BODY_2: type = BODY_ELLIP ; break ;
		case BODY_3: type = BODY_TRI   ; break ;
		case TAIL_0: break ;
		case TAIL_1: type = TAIL_LINE ; break ;
		case TAIL_2: type = TAIL_TRI  ; break ;
		case TAIL_3: type = TAIL_DOT  ; break ;
	}
	switch(parmblk->pb_obj)
	{
		case HEAD_0: case HEAD_1: case HEAD_2: case HEAD_3: 
			/*** radius to the right ***/
		    points[0] += (points[3]-points[1]) ;
		    points[2] += (points[3]-points[1]) ;
			break ;
		case TAIL_0: case TAIL_1: case TAIL_2: case TAIL_3: 
			/*** radius to the left ***/
		    points[0] -= (points[3]-points[1]) ;
		    points[2] -= (points[3]-points[1]) ;
			break ;
	}
	/* solid black */
	draw_note(p->handle, points, type, 8,BLACK,8,BLACK) ;

	/*** clipping = off ***/
	vs_clip(p->handle, FALSE, clipping_points) ;
	
	return(parmblk->pb_currstate) ;          
}


/***********************/
static void bold(void) /*
************************/
{
	/*** invert bits ***/ 
	ev_object[_BOLD].ob_state ^= CHECKED ;
	((PARAMS *)(ev_window->user))->effect ^= BOLD ; 

	/*** actualize protocol window ***/
	Awi_sendall(REDRAW, ev_window) ;
}

/************************/
static void light(void) /*
*************************/
{
	ev_object[_LIGHT].ob_state ^= CHECKED ;
	((PARAMS *)(ev_window->user))->effect ^= LIGHT ; 
	Awi_sendall(REDRAW, ev_window) ;
}
/*************************/
static void italic(void) /*
**************************/
{
	ev_object[_ITALIC].ob_state ^= CHECKED ;
	((PARAMS *)(ev_window->user))->effect ^= ITALIC ; 
	Awi_sendall(REDRAW, ev_window) ;
}
/*****************************/
static void underlined(void) /*
******************************/
{
	ev_object[_UNDERLINED].ob_state ^= CHECKED ;
	((PARAMS *)(ev_window->user))->effect ^= UNDERLINED ; 
	Awi_sendall(REDRAW, ev_window) ;
}
/***************************/
static void outlined(void) /*
****************************/
{
	ev_object[_OUTLINED].ob_state ^= CHECKED ;
	((PARAMS *)(ev_window->user))->effect ^= HOLLOW ; 
	Awi_sendall(REDRAW, ev_window) ;
}

/********************/
static void uncheck /*
*********************/
(
	OBJECT *obj	/* input: parent object */
)
{
	obj[RIEDER].ob_state     &= ~CHECKED ;
	obj[MIX].ob_state        &= ~CHECKED ;
	obj[BEYREUTHER].ob_state &= ~CHECKED ;
}
/*************************/
static void rieder(void) /*
**************************/
{
	uncheck(ev_object) ;
	ev_object[RIEDER].ob_state |= CHECKED ;
	Awi_sendall(REDRAW, ev_window) ;
}
/*****************************/
static void beyreuther(void) /*
******************************/
{
	uncheck(ev_object) ;
	ev_object[BEYREUTHER].ob_state |= CHECKED ;
	Awi_sendall(REDRAW, ev_window) ;
}
/**********************/
static void mix(void) /*
***********************/
{
	uncheck(ev_object) ;
	ev_object[MIX].ob_state |= CHECKED ;
	Awi_sendall(REDRAW, ev_window) ;
}

/*****************************/
static void show_fonts(void) /*
******************************/
{ 
PARAMS *p = ev_window->user ;
Awindow *font_wi = p->font_wi ;
int i ;
char *text ;

	if (font_wi == NULL)
	{
		/*** window must be created ***/
		p->font_wi = font_wi = Awi_create(&FONT_WINDOW) ;
		if (font_wi == NULL) return ;
		font_wi->user = p ;

		/*** copy filename ***/
		Ast_delete(font_wi->info) ;
		font_wi->info = Ast_create(ev_window->info) ;

		/*** adapt work object to number and names of fonts ***/
		for ( i = 0 ; i < p->number_fonts ; i++ )
		{
			text = font_wi->work[FONT_INDEX[i]].ob_spec.tedinfo->te_ptext ;
			strcpy(text, " ") ;
			strncat(text, (p->font_info + i)->font_name, MAXLEN) ;

			/*** select object according to font ID ***/
			if ( p->font_id == (p->font_info + i)->font_id )
				font_wi->work[FONT_INDEX[i]].ob_state |= SELECTED ;
			else font_wi->work[FONT_INDEX[i]].ob_state &= ~SELECTED ;
		}
		font_wi->work[FONT].ob_height = font_wi->work[FONT_0].ob_height * p->number_fonts ;

		/*** small window needs less elements ***/
		if (p->number_fonts <= 10) font_wi->wi_kind &= ~(VSLIDE|UPARROW|DNARROW|SIZER|FULLER) ;
	}
	Awi_show(font_wi) ; 
}

/***********************/
static void font(void) /*
************************/
{ 
int i ;
PARAMS *p = ev_window->user ;
	
	for ( i = 0 ; i < p->number_fonts ; i++ )
		if ( ev_window->work[FONT_INDEX[i]].ob_state & SELECTED )
			p->font_id = (p->font_info + i)->font_id ;
	Awi_sendall( REDRAW, p->params_wi ) ;	
}

 
/***********************/
static void close_font /*
************************/
(
	Awindow *wi /* input: font window */
)
{ ((PARAMS *)(wi->user))->font_wi = NULL ; }


/*******************************/
static void radio_button(void) /*
********************************/
{ 
	Awi_obchange(ev_window, EXAMPLE, -1) ;
	if (set_timer(ev_window) == -1) 
		Awi_sendall(REDRAW, ev_window) ; 
}


static char *popup = "0|1|2|3|4|5|6|7|8|9|A" ;

/*************************/
static void number(void) /*
**************************/
{
	PARAMS *p = ev_window->user ;
	static char *new_selection, 
	            *old_selection ;
	char *text ;

	old_selection = popup + (p->track_index << 1) ;
	new_selection = Ame_strpopup(ev_window, popup, old_selection, 40, -1 , -1) ;

	if (!new_selection) return ;

	if (*new_selection >= 'A')
	     p->track_index = *new_selection - 'A' + 10 ;
	else p->track_index = *new_selection - '0' ;

	Awi_obchange(ev_window, EXAMPLE, -1) ;
	
	/*** update text on number button ***/
    text = ev_object[NUMBER].ob_spec.tedinfo->te_ptext ;
	text[0] = popup[(p->track_index)<<1] ;
	text[1] = 0 ;
	Awi_obchange(ev_window, NUMBER, -1) ; 
}


static int /* return: 0 if success, -1 if file not found or wrong format */
/************/
load_params /*
*************/
(
	Awindow *wi,	/* input: parameters window */
	char *path,		/* input: path (including file name) */
	int general 	/* input: TRUE, when only general parameters shall be 
	                          loaded. If FALSE, all are loaded. */
)
{
	PARAMS *p = wi->user ;
	OBJECT *obj = wi->work ;
	Awindow *font_wi = p->font_wi ;
	long filesize ;
	PARAMS_FILE *pf ;
	char *text ;
	int i ;
	
	/*** load the parameter file into RAM ***/
	if ( load_file(path, &filesize, (void *)&pf) == file_loaded )
	{ 
		if ( (filesize == sizeof *pf) || 
		     (filesize == sizeof(PARAMS_FILE_V2)) /* backward compatibility */ ) 
		{
			/*** copy file data into object data ***/
			if (!general) 
			{
				/*** these parameters normally are indiviual for each piece of music ***/
				text = obj[TITLE].ob_spec.tedinfo->te_ptext ; 
				*text = '\0' ;
				strncat(text, pf->title, TITLE_FIELD_LENGTH) ;
				if (pf->bars_per_system > 99) pf->bars_per_system = 99 ; 
				if (pf->bar_length      > 99) pf->bar_length      = 99 ; 
				if (pf->sub_bars        >  9) pf->sub_bars        =  9 ; 
				itoa(pf->bars_per_system, obj[BARS_PER_SYSTEM].ob_spec.tedinfo->te_ptext, 10) ; 
				itoa(pf->bar_length     , obj[BAR_LENGTH     ].ob_spec.tedinfo->te_ptext, 10) ; 
				itoa(pf->sub_bars       , obj[SUB_BARS       ].ob_spec.tedinfo->te_ptext, 10) ; 
			}
			if (pf->left_border  > 99) pf->left_border  = 99 ; 
			if (pf->right_border > 99) pf->right_border = 99 ; 
			if (pf->upper_border > 99) pf->upper_border = 99 ; 
			if (pf->lower_border > 99) pf->lower_border = 99 ; 
			itoa(pf->left_border , obj[LEFT_BORDER ].ob_spec.tedinfo->te_ptext, 10) ; 
			itoa(pf->right_border, obj[RIGHT_BORDER].ob_spec.tedinfo->te_ptext, 10) ; 
			itoa(pf->upper_border, obj[UPPER_BORDER].ob_spec.tedinfo->te_ptext, 10) ; 
			itoa(pf->lower_border, obj[LOWER_BORDER].ob_spec.tedinfo->te_ptext, 10) ; 
			if (pf->system_distance > 99) pf->system_distance = 99 ; 
			if (pf->track_distance  > 99) pf->track_distance  = 99 ; 
			if (pf->note_distance   >  9) pf->note_distance   =  9 ; 
			if (pf->hor_lines       >  9) pf->hor_lines       =  9 ;
			itoa(pf->system_distance, obj[SYSTEM_DISTANCE].ob_spec.tedinfo->te_ptext, 10) ; 
			itoa(pf->track_distance , obj[TRACK_DISTANCE ].ob_spec.tedinfo->te_ptext, 10) ; 
			itoa(pf->note_distance  , obj[NOTE_DISTANCE  ].ob_spec.tedinfo->te_ptext, 10) ; 
			itoa(pf->hor_lines      , obj[HOR_LINES      ].ob_spec.tedinfo->te_ptext, 10) ;
			for (i = 0 ; i < 12 ; i++)
			{
				p->note_color [i]	  = pf->note_color [i] ;
				p->note_style [i]	  = pf->note_style [i] ; 
				p->note_color_ends[i] = pf->note_color_ends[i] ;
				p->note_style_ends[i] = pf->note_style_ends[i] ; 
				p->track_color[i]	  = pf->track_color[i] ;
				p->track_style[i]	  = pf->track_style[i] ;			
			}
			p->back_color = pf->back_color ;
			p->back_style = pf->back_style ;			
			if (filesize != sizeof(PARAMS_FILE_V2))
			{
				/*** new in V3: RGB values stored on disk ***/
				for (i = 0 ; i < 12 ; i++)
				{
					p->note_color [i]	  = rgb_to_index(p->note_color [i]) ;
					p->note_color_ends[i] = rgb_to_index(p->note_color_ends[i]) ;
					p->track_color[i]	  = rgb_to_index(p->track_color[i]) ;
				}
				p->back_color = rgb_to_index(p->back_color) ;
			}
			/*** deselect radio buttons ***/
			obj[RADIO_TAIL_0].ob_state &= ~SELECTED ;
			obj[RADIO_TAIL_1].ob_state &= ~SELECTED ;
			obj[RADIO_TAIL_2].ob_state &= ~SELECTED ;
			obj[RADIO_TAIL_3].ob_state &= ~SELECTED ;
			obj[RADIO_HEAD_0].ob_state &= ~SELECTED ;
			obj[RADIO_HEAD_1].ob_state &= ~SELECTED ;
			obj[RADIO_HEAD_2].ob_state &= ~SELECTED ;
			obj[RADIO_HEAD_3].ob_state &= ~SELECTED ;
			obj[RADIO_BODY_0].ob_state &= ~SELECTED ;
			obj[RADIO_BODY_1].ob_state &= ~SELECTED ;
			obj[RADIO_BODY_2].ob_state &= ~SELECTED ;
			obj[RADIO_BODY_3].ob_state &= ~SELECTED ;

			/*** determine radio button for note body ***/
			if      ( pf->note_type & BODY_TRI  ) obj[RADIO_BODY_3].ob_state |= SELECTED ;
			else if ( pf->note_type & BODY_ELLIP) obj[RADIO_BODY_2].ob_state |= SELECTED ;
			else if ( pf->note_type & BODY_RECT ) obj[RADIO_BODY_1].ob_state |= SELECTED ;
			else                                  obj[RADIO_BODY_0].ob_state |= SELECTED ;

			/*** determine radio button for note head ***/
			if      ( pf->note_type & HEAD_DOT ) obj[RADIO_HEAD_3].ob_state |= SELECTED ;
			else if ( pf->note_type & HEAD_TRI ) obj[RADIO_HEAD_2].ob_state |= SELECTED ;
			else if ( pf->note_type & HEAD_LINE) obj[RADIO_HEAD_1].ob_state |= SELECTED ;
			else                                 obj[RADIO_HEAD_0].ob_state |= SELECTED ;

			/*** determine radio button for note tail ***/
			if      ( pf->note_type & TAIL_DOT ) obj[RADIO_TAIL_3].ob_state |= SELECTED ;
			else if ( pf->note_type & TAIL_TRI ) obj[RADIO_TAIL_2].ob_state |= SELECTED ;
			else if ( pf->note_type & TAIL_LINE) obj[RADIO_TAIL_1].ob_state |= SELECTED ;
			else                                 obj[RADIO_TAIL_0].ob_state |= SELECTED ;

			/*** height and dynamic ***/
			if (pf->note_height > 9) pf->note_height = 9 ; 
			itoa(pf->note_height, obj[AVERAGE_HEIGHT].ob_spec.tedinfo->te_ptext, 10) ; 
			if (pf->note_dynamic > 9) pf->note_dynamic = 9 ; 
			itoa(pf->note_dynamic, obj[DYNAMIC].ob_spec.tedinfo->te_ptext, 10) ; 

			/*** fonts and headline style ***/
			for ( i = 0 ; i < p->number_fonts ; i++ )
			{
				if ( pf->font_id == (p->font_info + i)->font_id )
				{
					p->font_id = pf->font_id ;
					if (font_wi != NULL)
						font_wi->work[FONT_INDEX[i]].ob_state |= SELECTED ;
				}
				else
					if (font_wi != NULL)
						font_wi->work[FONT_INDEX[i]].ob_state &= ~SELECTED ;
			}
			p->effect = pf->effect ;
			if (p->effect & BOLD)       wi->menu[_BOLD]      .ob_state |= CHECKED ;
			else wi->menu[_BOLD]      .ob_state &= ~CHECKED ;
			if (p->effect & LIGHT)      wi->menu[_LIGHT]     .ob_state |= CHECKED ;
			else wi->menu[_LIGHT]     .ob_state &= ~CHECKED ;
			if (p->effect & ITALIC)     wi->menu[_ITALIC]    .ob_state |= CHECKED ;
			else wi->menu[_ITALIC]    .ob_state &= ~CHECKED ;
			if (p->effect & UNDERLINED) wi->menu[_UNDERLINED].ob_state |= CHECKED ;
			else wi->menu[_UNDERLINED].ob_state &= ~CHECKED ;
			if (p->effect & OUTLINED)   wi->menu[_OUTLINED]  .ob_state |= CHECKED ;
			else wi->menu[_OUTLINED]  .ob_state &= ~CHECKED ;

			/*** notation mode ***/
			wi->menu[BEYREUTHER].ob_state &= ~CHECKED ;
			wi->menu[RIEDER].ob_state     &= ~CHECKED ;
			wi->menu[MIX].ob_state        &= ~CHECKED ;
			switch (pf->mode) 
			{
				case Beyreuther: wi->menu[BEYREUTHER].ob_state |= CHECKED ; break ;
				case Mix:        wi->menu[MIX].ob_state        |= CHECKED ; break ;
				default:         wi->menu[RIEDER].ob_state     |= CHECKED ;
			}
		}
		else
		{

			/*** file verion is not 3.x or 2.x ***/ 
			alert_str(WRONG_FORMAT, path) ; 

			/*** free RAM file ***/
			unload_file(pf) ;
			return -1 ; /* no success */
		}
		/*** free RAM file ***/
		unload_file(pf) ;

		/*** refresh objects ***/
		Awi_obchange(wi, 0, -1) ;
		if (p->font_wi != NULL) Awi_obchange(p->font_wi, 0, -1) ;

		/*** actualize protocol window ***/
		Awi_sendall(REDRAW, wi) ;
		return 0 ; /* success */
	}
	return -1 ; /* no success */
}

/************************************/
static int get_bar_length(Awindow *wi)
/************************************/
{ return atoi(wi->work[BAR_LENGTH].ob_spec.tedinfo->te_ptext); }

/************************/
static void save_params /*
*************************/
(
	Awindow *wi,	/* input: parameters window */
	char *path,		/* input: path (including file name) */
	int ask_user    /* input: if TRUE, user is asked whether
					          modified parameters shall be saved */
)
{
	PARAMS_FILE pf, *pf_disk ;
	int i, handle, to_disk = TRUE ;
	long filesize ;
	char *filename ; /* without path */

	/*** search path for file name ***/
	filename = strrchr(path, '\\') ;
	if (filename) filename ++ ;
	else          filename = path ;

	/*** prepare structure for saving parameters ***/
	memset(&pf, 0, sizeof pf) ;
	get_style_params(wi,
	    pf.note_style     , pf.note_color, 
	    pf.note_style_ends, pf.note_color_ends, 
	  &(pf.note_type) , &(pf.note_height), &(pf.note_dynamic),
	    pf.track_style,   pf.track_color, 
	  &(pf.back_style), &(pf.back_color) 
	) ;
	/* transform to RGB colors */
	pf.back_color = index_to_rgb(pf.back_color) ;
	for (i = 0 ; i < 12 ; i++)
	{
		pf.note_color     [i] = index_to_rgb(pf.note_color     [i]) ;
		pf.note_color_ends[i] = index_to_rgb(pf.note_color_ends[i]) ;
		pf.track_color    [i] = index_to_rgb(pf.track_color    [i]) ;
	}
	strcpy(pf.title    , get_title(wi)) ;
	pf.bars_per_system = get_bars_per_system(wi) ;
	pf.bar_length      = get_bar_length(wi) ;
	pf.sub_bars        = get_bar_divider(wi) - 1 ;
	pf.left_border     = get_left_border(wi) ;
	pf.right_border    = get_right_border(wi) ;
	pf.upper_border    = get_upper_border(wi) ;
	pf.lower_border    = get_lower_border(wi) ;
	pf.system_distance = get_system_distance(wi) ;
	pf.track_distance  = get_track_distance(wi) ;
	pf.note_distance   = get_note_distance(wi) ; 
	pf.hor_lines       = get_hor_lines(wi) ;
	pf.font_id         = get_font_id(wi) ;
	pf.effect          = get_text_effects(wi) ;
	pf.mode            = get_mode(wi) ;
	
	if (ask_user)
	{
		if ( load_file(path, &filesize, (void *)&pf_disk) == file_loaded )
		{
			if ( filesize != sizeof(pf) ||
			      memcmp(&pf, pf_disk, sizeof(pf)) ) 
			{
				if (alert_str(SAVE_PARAMS, filename) != 1)
					to_disk = FALSE ; /* user doesn't want to save changes */
			}
			else to_disk = FALSE ; /* nothing has changed */
			unload_file(pf_disk) ;
		}
	}
	if (to_disk)
	{
		/*** open file ***/
		handle = (int)Fopen(path, FO_WRITE) ;
		if ( handle < 0 )
		{
			/*** try to create ?.PAR ***/
			handle = (int)Fcreate(path, 0 /* ordinary file */) ;
			if ( handle < 0 )
			{
				alert_str(NOT_CREATED, path) ; 
				return ;
			}
		}
		/*** save structure ***/
		if ( Fwrite(handle, sizeof pf, &pf) < 0 )
			alert_str(WRITE_ERROR, path) ; 
		/*** close file ***/
		Fclose(handle) ;
	}
}
/*******************/
static void save() /*
********************/
{ save_params(ev_window, ((PARAMS *)(ev_window->user))->path, FALSE) ; }

/*******************/
static void load() /*
********************/
{ load_params(ev_window, ((PARAMS *)(ev_window->user))->path, FALSE) ; } 

/****************************/
static void load_defaults() /*
*****************************/
{ load_params(ev_window, default_path, TRUE) ; } 


/**********************/
static void save___() /*
***********************/
{ 
char filename[filename_length] ; /* here: a dummy */

	switch ( get_file_name(savepathname, savepath, filename, "Save parameter file") ) 
	{
	case no_file_selected:  
	case file_select_error: return ;
	}
	save_params(ev_window, savepathname, FALSE) ; 
}
/**********************/
static void load___() /*
***********************/
{ 
char filename[filename_length] ; /* here: a dummy */

	switch ( get_file_name(loadpathname, loadpath, filename, "Load parameter file") ) 
	{
		case no_file_selected:  
		case file_select_error: return ;
	}
	load_params(ev_window, loadpathname, FALSE) ; 
} 

/*****************/
static void keys /*
******************/
(
	Awindow *wi,	/* input: parameters window */
	int kstate, 	/* input: see ACS manual */
	int key			/* input: ASCII code and scan code */
)
{
	if (set_timer(wi) == -1) Awi_sendall(REDRAW, wi) ;

	/*** forward event to ACS for further treatment (e.g. by the desktop) ***/
	Awi_keys(wi, kstate, key) ;
}


static int /* return: TRUE if task was treated, FALSE else */
/********/
service /*
*********/
(
	Awindow *wi, 	/* input: parameters window */
	int task, 		/* input: message from ACS or other window */
	void *not_used  /* input: not used here */
)
{
	switch (task)
	{	
	case AS_TERM: Awi_sendall(TERMINATE, wi) ; break ; 
	case AS_INFO: A_dialog(&PARAMS_INFO) ; break ;
	default:      return FALSE ;
	}
	return TRUE ;
}

/******************************************************************
*
*             PUBLISHED INTERFACE
*
******************************************************************/

/***********************/
void PARAMS_destructor /*
************************/
(
	Awindow *wi /* input: parameters window to destroy */
)
{
	PARAMS *p ; 

	if (wi) 
	{
		p = wi->user ;
		if (p) 
		{
			/*** if parameters have been changed, 
			     ask user [and save if desired] ***/ 
			save_params(wi, p->path, TRUE) ; 

			if (p->font_wi)
			{
				memory_symptom = "PaDstr font_wi" ;
				Awi_delete( p->font_wi ) ;
			}
			memory_symptom = "PaDstr p" ;
			My_free(p) ; 	
		}
		memory_symptom = "PaDstr params_wi" ;
		Awi_delete(wi) ; 
	}
}

Awindow *	/* return: NULL if no success, parameters window else */
/*******************/
PARAMS_constructor /*
********************/
(
	PARAMS_INIT *params_init /* input: init values */
)
{
	Awindow *wi ;
	PARAMS *p ;
	char *last_point ;
	int i ;

	/*** enter filename into icon text and info line, disable menu entries if necessary ***/
	PARAMS_WINDOW.iconblk = params_init->icon ;
	PARAMS_WINDOW.iconblk->monoblk.ib_ptext = params_init->filename ;
	PARAMS_WINDOW.info = params_init->filename ;

	/*** create the parameters window ***/
	wi = Awi_create(&PARAMS_WINDOW) ;
	if (!wi) return NULL ;

	/*** allocate params structure and initialize ***/
	wi->user = p = My_alloc(sizeof *p) ;
	if (!p) { PARAMS_destructor(wi) ; return NULL ; }
	memset(p, 0, sizeof *p) ;
	p->handle = params_init->handle ;
	for (i = 0 ; i < 12 ; i++)
	{
		p->note_style     [i] = IP_SOLID ; p->note_color     [i] = BLACK ; 
		p->note_style_ends[i] = IP_SOLID ; p->note_color_ends[i] = BLACK ; 
		p->track_style    [i] = IP_1PATT ; p->track_color    [i] = BLACK ; 
	}
	p->back_style = 0 ; 
	p->back_color = WHITE ; 	
	p->style_to_update = &(p->back_style) ;
	p->color_to_update = &(p->back_color) ;
	p->effect = NORMAL ;
	p->params_wi = wi ;
	p->font_info = params_init->font_info ;
	p->number_fonts = params_init->number_fonts ;
	strcpy(p->path, params_init->path) ;
	strcpy(loadpath, params_init->path) ;
	*( strrchr(p->path, '\\') + 1 ) = '\0' ;
	*( strrchr(loadpath, '\\') + 1 ) = '\0' ;
	strcat(p->path, params_init->filename) ;
	strcat(loadpath, "*.PAR") ;
	strcpy(savepath, loadpath) ;

	/*** extension ".PAR" for parameters filename ***/
	last_point = strrchr(p->path, '.') ;   
	if (last_point != NULL) *last_point = '\0' ;
	strcat(p->path, ".PAR") ;

	/*** default path name (if no own parameters file exists for the MIDI file) ***/  
	strcpy(default_path, apppath) ;
	*( strrchr(default_path, '\\') + 1 ) = 0 ;
	strcat(default_path, "MIDI_PRT.PAR") ;

	/*** load parameters file ***/		
	if ( load_params(wi, p->path, FALSE) < 0 ) 
		 load_params(wi, default_path, FALSE) ;

	/*** if headline still empty: enter filename into headline ***/
	if ( *(wi->work[TITLE].ob_spec.tedinfo->te_ptext) == '\0' )
		strncat(wi->work[TITLE].ob_spec.tedinfo->te_ptext, params_init->filename, TITLE_FIELD_LENGTH) ;

	/*** userdef drawing routines called by AES need access to PARAMS ***/ 
	wi->work[EXAMPLE].ob_spec.userblk->ub_parm =
	wi->work[TAIL_0 ].ob_spec.userblk->ub_parm =
	wi->work[TAIL_1 ].ob_spec.userblk->ub_parm =
	wi->work[TAIL_2 ].ob_spec.userblk->ub_parm =
	wi->work[TAIL_3 ].ob_spec.userblk->ub_parm =
	wi->work[HEAD_0 ].ob_spec.userblk->ub_parm =
	wi->work[HEAD_1 ].ob_spec.userblk->ub_parm =
	wi->work[HEAD_2 ].ob_spec.userblk->ub_parm =
	wi->work[HEAD_3 ].ob_spec.userblk->ub_parm =
	wi->work[BODY_0 ].ob_spec.userblk->ub_parm =
	wi->work[BODY_1 ].ob_spec.userblk->ub_parm =
	wi->work[BODY_2 ].ob_spec.userblk->ub_parm =
	wi->work[BODY_3 ].ob_spec.userblk->ub_parm = (long)p ;

	/*** return success message ***/
	return wi ;
}
#endif /* ifdef (__PUREC__) */


char *get_title(Awindow *wi) 
{
#if defined (__PUREC__)	
	return wi->work[TITLE].ob_spec.tedinfo->te_ptext ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetHeadline() ;
#endif
}

int get_bars_per_system(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[BARS_PER_SYSTEM].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetBarsPerSystem() ;
#endif
}

int get_system_length(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return ( get_bar_length(wi) * get_bars_per_system(wi) ) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetBarLength() * ((MIDI_OPTIONS *)wi)->GetBarsPerSystem() ;
#endif
}

char get_bar_divider(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[SUB_BARS].ob_spec.tedinfo->te_ptext) + 1 ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetSubBars() + 1 ;
#endif
}

int get_left_border(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[LEFT_BORDER].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetLeftBorder() ;
#endif
}

int get_right_border(Awindow *wi)
{
#if defined (__PUREC__)	
	return atoi(wi->work[RIGHT_BORDER].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetRightBorder() ;
#endif
}

int get_upper_border(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[UPPER_BORDER].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetUpperBorder() ;
#endif
}

int get_lower_border(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[LOWER_BORDER].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetLowerBorder() ;
#endif
}

int get_system_distance(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[SYSTEM_DISTANCE].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetSystemDistance() ;
#endif
}

int get_track_distance(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[TRACK_DISTANCE].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetTrackDistance() ;
#endif
}

int get_note_distance(Awindow *wi)
{ 
#if defined (__PUREC__)	
	return atoi(wi->work[NOTE_DISTANCE].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetNoteDistance() ;
#endif
}

int get_hor_lines(Awindow *wi)
{
#if defined (__PUREC__)	
	return atoi(wi->work[HOR_LINES].ob_spec.tedinfo->te_ptext) ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetHorLines() ;
#endif
}

/**********************/
void get_style_params /*
***********************/
(
	Awindow *wi, 		/* input: parameters window */
    int note_style[12],	/* output: 12 note styles */
    int note_color[12],	/* output: 12 note colors */
    int note_style_ends[12],	/* output: 12 note end styles */
    int note_color_ends[12],	/* output: 12 note end colors */
    int *note_type,			/* output: note type */
    char *note_height, 		/* output: note height */
    char *note_dynamic,   	/* output: note dynamic */
    int track_style[12], /* output: 12 staff styles */
    int track_color[12], /* output: 12 staff colors */
    int *back_style,	/* output: background style */
    int *back_color		/* output: background color */
) 
{
#if defined (__PUREC__)	
	PARAMS *p = wi->user ;
	int i ;

	/*** determine note type ***/
	*note_type = 0 ;
	if (wi->work[RADIO_HEAD_1].ob_state & SELECTED) *note_type |= HEAD_LINE ;
	if (wi->work[RADIO_HEAD_2].ob_state & SELECTED) *note_type |= HEAD_TRI ;
	if (wi->work[RADIO_HEAD_3].ob_state & SELECTED) *note_type |= HEAD_DOT ;
	if (wi->work[RADIO_BODY_1].ob_state & SELECTED) *note_type |= BODY_RECT ;
	if (wi->work[RADIO_BODY_2].ob_state & SELECTED) *note_type |= BODY_ELLIP ;
	if (wi->work[RADIO_BODY_3].ob_state & SELECTED) *note_type |= BODY_TRI ;
	if (wi->work[RADIO_TAIL_1].ob_state & SELECTED) *note_type |= TAIL_LINE ;
	if (wi->work[RADIO_TAIL_2].ob_state & SELECTED) *note_type |= TAIL_TRI ;
	if (wi->work[RADIO_TAIL_3].ob_state & SELECTED) *note_type |= TAIL_DOT ;

	/*** rest ***/
	for (i = 0 ; i < 12 ; i++)
	{
		note_color     [i] = p->note_color [i] ;
		note_style     [i] = p->note_style [i] ;
		note_color_ends[i] = p->note_color_ends [i] ;
		note_style_ends[i] = p->note_style_ends [i] ;
		track_color    [i] = p->track_color[i] ;
		track_style    [i] = p->track_style[i] ;
	}
	*back_color = p->back_color ;
	*back_style = p->back_style ;
	*note_height  = (char)atoi(wi->work[AVERAGE_HEIGHT].ob_spec.tedinfo->te_ptext) ;
	*note_dynamic = (char)atoi(wi->work[DYNAMIC].ob_spec.tedinfo->te_ptext) ; 
#else
	((MIDI_OPTIONS *)wi)->GetStyleParams (
		 note_style     , note_color,
		 note_style_ends, note_color_ends,
		 note_type,note_height,note_dynamic,
		 track_style,track_color,
		 back_style, back_color	
	) ;
#endif
}


int /* return: VDI text effects for headline */
/*****************/
get_text_effects /*
******************/
(
	Awindow *wi /* input: parameters window */
) 
{ 
#if defined (__PUREC__)	
	return ((PARAMS *)(wi->user))->effect ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetTextEffects() ;
#endif
}

int /* return: GDOS font ID */
/************/
get_font_id /*
*************/
(
	Awindow *wi	/* input: parameters window */
)
{ 
#if defined (__PUREC__)	
	return ((PARAMS *)(wi->user))->font_id ; 
#else
	return ((MIDI_OPTIONS *)wi)->GetFontID() ;
#endif
}

int	/* return: notation mode */
/*********/
get_mode /*
**********/
(
	Awindow *wi /* input: parameters window */
)
{ 
#if defined (__PUREC__)	
	if (wi->menu[BEYREUTHER].ob_state & CHECKED) return Beyreuther ;
	if (wi->menu[MIX].ob_state        & CHECKED) return Mix ;
	return Rieder ;
#else
	return ((MIDI_OPTIONS *)wi)->GetMode() ;
#endif
}
