/**************************************************************
*
*                PARAMS.HPP
*
**************************************************************/

#define Beyreuther 0x20    /* notation mode Beyreuther */
#define Rieder     0x40    /* notation mode Rieder     */
#define Mix        0x30    /* notation mode Mix        */

#if defined (__PUREC__)
#define MAX_FONTS 128      /* maximum number of GDOS fonts that can be */
                           /* managed by this module                   */
#define MAXLEN 32          /* length of GDOS font name */
typedef struct 
{ 
	int font_id ; 
	char font_name[MAXLEN + 1] ; 
}
FONT_INFO ; 


typedef struct 
{
	CICONBLK *icon ;		/* icon to use for ACS window */
	char *filename ;		/* of MIDI file */
	char *path ;
	int handle ;			/* VDI screen handle to use */
	FONT_INFO *font_info ;  /* array of see above */
	int number_fonts ;      /* available VDI fonts */
} 
PARAMS_INIT ;

/*******************************************************************
*/
	Awindow *PARAMS_constructor(PARAMS_INIT *params_init) ;
/*
* Constructor of the PARAMS object. It
* constructs an object instance of the PARAMS object containing
* ACS object and own data instances.
* 
* OUTPUT:  params icon placed on screen and
*          own params data instance connected to ACS-window
*          instance
* RETURN:  errors: NULL or -if successfull- ACS-window pointer,
*          which will be used as handle for the object instance
* 
*******************************************************************/


/*******************************************************************
*/
void PARAMS_destructor(Awindow *window) ;
/*
* Destructor of the PARAMS object. It
* frees all memory used by ACS and own data instances.
* 
*******************************************************************/
#endif

/*******************************************************************
*/
	int get_left_border(Awindow *) ;     /* left border in % from sheet width */
	int get_right_border(Awindow *) ;    /* right ... */
	int get_upper_border(Awindow *) ;    /* upper border in % from sheet height */
	int get_lower_border(Awindow *) ;    /* lower ... */
	int get_system_distance(Awindow *) ; /* note system distance in % from sheet height */
	int get_track_distance(Awindow *) ;  /* track distance ... */
	int get_note_distance(Awindow *) ;   /* succeeding notes' distance in %% from sheet height */
	int get_bars_per_system(Awindow *) ; /* bars per note system (line) */
	int get_system_length(Awindow *) ;   /* system length in 1/8 beats */
	char get_bar_divider(Awindow *) ;    /* sub-bars per bar */
	int get_text_effects(Awindow *) ;    /* VDI text effects for headline */
	int get_mode(Awindow *) ;            /* notation mode: Rieder, Beyreuther, ... */
	int get_font_id(Awindow *) ;         /* VDI font for all texts */
	int get_hor_lines(Awindow *wi) ;     /* horizontal lines per dodecime */
	char *get_title(Awindow *) ;         /* headline text */
/*
*   "get_..." are simple access procedures.
* 
*******************************************************************/


/*******************************************************************
*/
	void get_style_params(
		Awindow *window,   /* in: params window instance */
		int style[12],     /* out: 12 styles for note body */
		int color[12],     /* out: 12 colors for note body */
		int style_ends[12],/* out: 12 styles for note ends */
		int color_ends[12],/* out: 12 colors for note ends */
		int *type,         /* out: of note body, left end, right end (mask) */
		char *height,       /* out: average height of note (relative 0..9) */
		char *dynamic,      /* out: effect of note-on dynamic on note height (relative 0..9) */
		int staff_style[12],/* out: 12 styles for staves */
		int staff_color[12],/* out: 12 colors for staves */
		int *back_style,	/* out: style for system background */
		int *back_color		/* out: color for system background */
	) ;
/*
*   access procedure to all parameters 
*   affecting the drawing of the notes themselves. 
* 
*******************************************************************/