/**************************************************************
*
*                GRAPHIC.C
*
**************************************************************/

#include <my_vdi.h> // before acs.h
#if defined (__PUREC__)
#	include <acs.h>
#endif
#include <stdlib.h>
#include <graphic.h>

#define ELL_STRETCH 4/3   /* manipulation of ellipses in y-direction */
                          /* 2 * manipulation of circle radii, too */
#define TRI_STRETCH 4/3   /* manipulation of triangles in y-direction */

/******** auxiliary procedures *********/

static void 
/***********/
draw_head /*
************/
(
	int handle, 
	int *p /*[6]*/, 
	int y_mean, 
	int dy, 
	int type
)
{
	if (type & HEAD_TRI)  
	{
		v_fillarea(handle, 3, p) ; 
		return ; 
	}
	if (type & HEAD_LINE) 
	{
		v_pline(handle, 2, p) ; 
		return ;
	}
	if (type & HEAD_DOT)
		v_circle(handle, p[0], y_mean, dy * ELL_STRETCH/2) ;
}	

static void 
/***********/
draw_tail /*
************/
(
	int handle, 
	int *p /*[6]*/, 
	int y_mean, 
	int dy, 
	int type
)
{
	if (type & TAIL_TRI)  
	{
		v_fillarea(handle, 3, p) ; 
		return ; 
	}
	if (type & TAIL_LINE) 
	{
		v_pline(handle, 2, p) ; 
		return ;
	}
	if (type & TAIL_DOT)
		v_circle(handle, p[0], y_mean, dy * ELL_STRETCH/2) ;
}	

static void 
/************/
body_ellip /*
*************/
(
	int handle, 
	int *points /*[4]*/, 
	int y_mean, 
	int dy_ell, 
	int type
) 
{
	if (type & HEAD_CUT) 
	{
		v_ellpie(handle, points[0], y_mean, points[2]-points[0], dy_ell, 2700,900) ;
		return ;
	}
	if (type & TAIL_CUT)
	{
		v_ellpie(handle, points[2], y_mean, points[2]-points[0], dy_ell, 900,2700) ;
		return ;
	}
	v_ellipse(handle, (points[0]+points[2])>>1, y_mean, (points[2]-points[0])>>1, dy_ell) ;
}

				
static void 
/**********/
body_tri /*
***********/
(
	int handle, 
	int *points /*[4]*/, 
	int y_mean,
	int dy,
	int type
) 
{
	int p[8] ;
	
	dy = dy * TRI_STRETCH ;
	p[0] = p[2] = points[0] ;
	p[4] = p[6] = points[2] ;
	if (type & HEAD_CUT) 
	{
		p[1] = y_mean - (dy >> 1) ;
		p[3] = y_mean + (dy >> 1) ;
		p[5] = y_mean ;
		v_fillarea(handle, 3, p) ;
		return ;
	}
	if (type & TAIL_CUT)
	{
		p[1] = y_mean - dy ;
		p[3] = y_mean + dy ;
		p[5] = y_mean - (dy >> 1) ;
		p[7] = y_mean + (dy >> 1) ;
		v_fillarea(handle, 4, p) ;
		return ;
	}
	p[1] = y_mean - dy ;
	p[3] = y_mean + dy ;
	p[5] = y_mean ;
	v_fillarea(handle, 3, p) ;
}				


static void
/*********************/
set_style_and_color /*
**********************/
(
	int handle,
	int style,
	int color
)
{
	if (!style) vsf_interior(handle, FIS_HOLLOW) ;
	else        vsf_interior(handle, FIS_PATTERN) ;
	vsf_style(handle, style) ;
	vsf_color(handle, color) ; 
	vsl_color(handle, color) ; 
	vst_color(handle, color) ;
}

void
/*****************/
draw_background /*
******************/
(
	int handle, 	/* VDI workstation handle */
	int *points,	/* points[0]:     x-value of 1. corner
   					   points[1]:     y-value of 1. corner
	   				   points[2]:     x-value of 2. corner
		   			   points[3]:     y-value of 2. corner */
	int style, 		/* fill style */
	int color		/* fill	color */
)
{
	vswr_mode(handle, MD_REPLACE) ;
	set_style_and_color(handle, style, color) ;
	vr_recfl(handle, points) ;
}
	
void
/***********/
draw_note /*
************/
(
	int handle, 	/* VDI workstation handle */
	int *points,	/* points[0]:     x-value of 1. corner
   					   points[1]:     y-value of 1. corner
	   				   points[2]:     x-value of 2. corner
		   			   points[3]:     y-value of 2. corner */
	int type,       /* see below */
	int style, 		/* fill style body */
	int color,		/* fill	color body */
	int end_style,	/* fill style ends */
	int end_color	/* fill	color ends */
)
{
	const int dy     = (points[3] - points[1])>>1, 
	          dy_ell = (int)( (float)dy * ELL_STRETCH ),
	          dy_abs = abs(points[3] - points[1]) ;
	const int y_mean = (points[3] + points[1])>>1 ;
	int p[6] ;

	vswr_mode(handle, MD_REPLACE) ;
	vsf_perimeter(handle, 1) ; /* with perimeter */
	vsl_type(handle, SOLID) ;  /* solid line */
	
	p[1] = points[1] - dy ; 
	p[3] = points[3] + dy ;
	p[5] = y_mean ;
	
	/*** draw triangle ends in the background ***/
	set_style_and_color(handle, end_style, end_color) ;
	if (type & HEAD_TRI)
	{	
		p[0] = p[2] = points[0] ; 
		p[4] = p[0] + dy_abs ;
		draw_head(handle, p, y_mean, dy, type) ;
	}
	if (type & TAIL_TRI)
	{
		p[0] = p[2] = points[2] ; 
		p[4] = p[0] - dy_abs ;
		draw_tail(handle, p, y_mean, dy, type) ;
	}
	
	/*** draw note body ***/
	set_style_and_color(handle, style, color) ;
	if      (type & BODY_ELLIP) body_ellip(handle, points, y_mean, dy_ell, type) ;
	else if (type & BODY_RECT)  v_bar     (handle, points) ;
	else if (type & BODY_TRI)   body_tri  (handle, points, y_mean, dy, type) ;

	/*** draw line or dot ends in the foreground ***/
	set_style_and_color(handle, end_style, end_color) ;
	if (type & (HEAD_DOT | HEAD_LINE))
	{	
		p[0] = p[2] = points[0] ; 
		p[4] = p[0] + dy_abs ;
		draw_head(handle, p, y_mean, dy, type) ;
	}	
	if (type & (TAIL_DOT | TAIL_LINE))
	{
		p[0] = p[2] = points[2] ;
		p[4] = p[0] - dy_abs ;
		draw_tail(handle, p, y_mean, dy, type) ;
	}
}

void
/************/
draw_lines /*
*************/
(
	int handle, 			/* VDI workstation handle */
	char number,			/* number of lines to be drawn */
	char first_marked, 		/* first line with special marking */
	char d_marked, 			/* special marking each d_marked lines */
	MARK_MODE mode, 		/* where to place rectangle marks */
	int styles[12], 		/* styles to be used after a special line */
	int colors[12], 		/* colors to be used after a special line */
	char first_number,      /* number (text) for first special line */
	int number_dots,		/* of special line, no dots if -1 */
	int dx_text, 			/* text (number) offset from x_start on */
	int dy_text,			/* text (number) offset from y_start on */
	int x_start, 			/* of first line */
	int y_start,			/* of first line */
	int x_end, 				/* of first line */
	int y_end,				/* of first_line */
	float dx, 				/* distance of succeeding lines */
	float dy				/* distance of succeeding lines */
) 
{
	char i, number_offset, k ; 
	int points[4], dx_m = (int)(dx * d_marked / 12 + 0.5) , 
	               dy_m = (int)(dy * d_marked / 12 + 0.5) ; 
	int dx_int = (int)(dx + 0.5), dy_int ;
	int x_width, y_width, color, style ;
	char *dummy = "--" ; 
	
	if (dy >= 0) dy_int = (int)(dy + 0.5) ;
	else         dy_int = (int)(dy - 0.5) ; /* stupid processors! */
	vswr_mode(handle, MD_REPLACE) ;
	vsf_perimeter(handle, 0) ;

	number_offset = 11 ;
	color = colors[(first_number + number_offset) % 12] ;
	vsl_color(handle, color) ;
	number_offset = -1 ;
	
	for (i = 0 ; i < number ; i ++ )
	{
		points[0] = (int)(dx * i + 0.5) + x_start ; 
		if (dy > 0) points[1] = (int)(dy * i + 0.5) + y_start ;
		else        points[1] = (int)(dy * i - 0.5) + y_start ;
		points[2] = points[0] + x_end - x_start ; 
		points[3] = points[1] + y_end - y_start ;

		if (i % d_marked != first_marked) v_pline(handle, 2, points) ;
		else 
		{
			/*** switch to next color and style ***/
			number_offset ++ ;
			color = colors[(first_number + number_offset) % 12] ;
			style = styles[(first_number + number_offset) % 12] ;
			set_style_and_color(handle, style, color) ;
	
			if (first_marked == -1) v_pline(handle, 2, points) ;
			else
			{
				/*** mark with number ***/
				if ( dx_text != INVALID_DX && dy_text != INVALID_DY )
					v_gtext( handle, points[0] + dx_text, points[1] + dy_text, 
					         itoa(first_number + number_offset, dummy, 12) ) ;
				         
				if (style == IP_HOLLOW) vsl_width(handle, 3) ;
				if (number_dots <= 0)
					if (mode == behind)
					{
						/*** mark non-dotted, rectangle behind line ***/
						points[0] += dx_m ; points[1] += dy_m ; points[2] -= dx_m ; points[3] -= dy_m ; 
						if (style != IP_HOLLOW) v_bar(handle, points) ;
						
						points[0] -= dx_m ; points[1] -= dy_m ; points[2] += dx_m ; points[3] += dy_m ; 
						v_pline(handle, 2, points) ;					
					}
					else 
					{
						/*** mark non-dotted, rectangle between lines ***/
						points[0] += dx_int ; 
						points[1] += dy_int ;
						if (i != number - 1) if (style != IP_HOLLOW) v_bar(handle, points) ;

						points[0] -= dx_int ; 
						points[1] -= dy_int ;
						v_pline(handle, 2, points) ;					
					}
				else 
				{
					x_width = ((x_end - x_start) / number_dots) >> 1 ;
					y_width = ((y_end - y_start) / number_dots) >> 1 ;
					for (k = 0 ; k < number_dots ; k++)
					{
						/*** draw dot ***/
						points[0] = (int)(dx * i + x_start 
						            + (float)(x_end - x_start) * k / number_dots + dx_m + 0.5) ; 
						points[1] = (int)(dy * i + y_start 
						            + (float)(y_end - y_start) * k / number_dots + dy_m + 0.5) ;
						points[2] = points[0] + x_width - (dx_m << 1) ;
						points[3] = points[1] + y_width - (dy_m << 1) ;
						if (style != IP_HOLLOW) v_bar(handle, points) ;
					
						points[0] -= dx_m ; points[1] -= dy_m ; points[2] += dx_m ; points[3] += dy_m ; 
						v_pline(handle, 2, points) ;					
					}
				}
				/*** reset line width ***/
				vsl_width(handle, 1) ;
			}
		}
	}
}


static int rgb_colors[MAX_COLORS] ;

void
/****************/
init_rgb_table /*
*****************/
(int handle)
{
	int i, rgb[3] ;
	for (i = 0 ; i < MAX_COLORS && i < ncolors ; i++)
	{
		vq_color(handle, i, 1, rgb);
		rgb_colors[i] = 
			 thousand_to_5bit(rgb[0])      +
			(thousand_to_5bit(rgb[1])<<5 ) +
			(thousand_to_5bit(rgb[2])<<10) ;
	}
}

/******************/
int rgb_to_index /*
*******************/
(int rgb)
{
	int i, diff, best_i = 0, smallest_diff = 0x7FFF ;
	for (i = 0 ; i < MAX_COLORS && i < ncolors ; i++)
	{
		diff = 
			 abs((rgb_colors[i] & RGB_RED  ) - (rgb & RGB_RED  )) +
			(abs((rgb_colors[i] & RGB_GREEN) - (rgb & RGB_GREEN))>>5) +
			(abs((rgb_colors[i] & RGB_BLUE ) - (rgb & RGB_BLUE ))>>10) ;
		if (diff < smallest_diff)
		{
			best_i = i ; 
			smallest_diff = diff ;
		}
	}
	return best_i ;
}

/******************/
int index_to_rgb /*
*******************/
(int index)
{
	if (index > ncolors    - 1 ||
	    index > MAX_COLORS - 1)
		index = BLACK ;
	return rgb_colors[index] ;
}